Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
102 lines
2.7 KiB
Plaintext
102 lines
2.7 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDA Experimental in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#ifndef __COMMON_HOST_DEVICE_H__
|
|
#define __COMMON_HOST_DEVICE_H__
|
|
|
|
#include "utility.cuh"
|
|
|
|
template <typename Dims, typename Lambda>
|
|
void __global__ lambda_launcher(const Dims dims, const Lambda lambda)
|
|
{
|
|
lambda(dims);
|
|
}
|
|
|
|
template <typename Comparator, unsigned int FilterArch>
|
|
bool arch_filter(const cudaDeviceProp& props)
|
|
{
|
|
int act_arch = props.major * 10 + props.minor;
|
|
if (Comparator()(act_arch, FilterArch))
|
|
{
|
|
return true;
|
|
}
|
|
return false;
|
|
}
|
|
|
|
static bool skip_host_exec(bool (* /* filter */)(const cudaDeviceProp&))
|
|
{
|
|
return false;
|
|
}
|
|
|
|
static bool skip_device_exec(bool (*filter)(const cudaDeviceProp&))
|
|
{
|
|
cudaDeviceProp props;
|
|
REQUIRE_CUDART(cudaGetDeviceProperties(&props, 0));
|
|
return filter(props);
|
|
}
|
|
|
|
template <typename Dims, typename Lambda, typename... Filters>
|
|
void test_host_dev(const Dims& dims, const Lambda& lambda, const Filters&... filters)
|
|
{
|
|
SECTION("Host execution")
|
|
{
|
|
if ((... && !skip_host_exec(filters)))
|
|
{
|
|
// host testing
|
|
lambda(dims);
|
|
}
|
|
}
|
|
|
|
SECTION("Device execution")
|
|
{
|
|
// Asymmetrical but cleaner
|
|
if ((... || skip_device_exec(filters)))
|
|
{
|
|
return;
|
|
}
|
|
|
|
cudaLaunchConfig_t config = {};
|
|
config.gridDim = {0};
|
|
cudaLaunchAttribute attrs[1];
|
|
config.attrs = &attrs[0];
|
|
|
|
config.blockDim = dims.extents(cuda::gpu_thread, cuda::block);
|
|
config.gridDim = dims.extents(cuda::block, cuda::grid);
|
|
|
|
if constexpr (Dims::has_level(cluster))
|
|
{
|
|
dim3 cluster_dims = dims.extents(cuda::block, cuda::cluster);
|
|
config.attrs[config.numAttrs].id = cudaLaunchAttributeClusterDimension;
|
|
config.attrs[config.numAttrs].val.clusterDim = {cluster_dims.x, cluster_dims.y, cluster_dims.z};
|
|
config.numAttrs = 1;
|
|
}
|
|
else
|
|
{
|
|
config.numAttrs = 0;
|
|
}
|
|
|
|
// device testing
|
|
REQUIRE_CUDART(cudaLaunchKernelEx(&config, lambda_launcher<Dims, Lambda>, dims, lambda));
|
|
REQUIRE_CUDART(cudaDeviceSynchronize());
|
|
}
|
|
}
|
|
|
|
template <typename Fn, typename Tuple>
|
|
void apply_each(const Fn& fn, const Tuple& tuple)
|
|
{
|
|
cuda::std::apply(
|
|
[&](const auto&... elems) {
|
|
(fn(elems), ...);
|
|
},
|
|
tuple);
|
|
}
|
|
|
|
#endif // __COMMON_HOST_DEVICE_H__
|